@@ -2,7 +2,7 @@
 /*-
  * Copyright (c) 2005-2007 Benedikt Meurer <benny@xfce.org>
  * Copyright (c) 2005      Jeff Franks <jcfranks@xfce.org>
- * Copyright (c) 2009      Jannis Pohlmann <jannis@xfce.org>
+ * Copyright (c) 2009-2010 Jannis Pohlmann <jannis@xfce.org>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -36,6 +36,10 @@
 #include <time.h>
 #endif
 
+#ifdef HAVE_GUDEV
+#include <gudev/gudev.h>
+#endif
+
 #include <libxfce4ui/libxfce4ui.h>
 
 #include <thunar/thunar-application.h>
@@ -94,14 +98,10 @@ static void           thunar_application_launch                 (ThunarApplicati
                                                                  GClosure               *new_files_closure);
 static void           thunar_application_window_destroyed       (GtkWidget              *window,
                                                                  ThunarApplication      *application);
-static void           thunar_application_drive_connected        (GVolumeMonitor         *volume_monitor,
-                                                                 GDrive                 *drive,
-                                                                 ThunarApplication      *application);
-static void           thunar_application_drive_disconnected     (GVolumeMonitor         *volume_monitor,
-                                                                 GDrive                 *drive,
-                                                                 ThunarApplication      *application);
-static void           thunar_application_drive_eject            (GVolumeMonitor         *volume_monitor,
-                                                                 GDrive                 *drive,
+#ifdef HAVE_GUDEV
+static void           thunar_application_uevent                 (GUdevClient            *client,
+                                                                 const gchar            *action,
+                                                                 GUdevDevice            *device,
                                                                  ThunarApplication      *application);
 static gboolean       thunar_application_volman_idle            (gpointer                user_data);
 static void           thunar_application_volman_idle_destroy    (gpointer                user_data);
@@ -109,6 +109,7 @@ static void           thunar_application_volman_watch           (GPid
                                                                  gint                    status,
                                                                  gpointer                user_data);
 static void           thunar_application_volman_watch_destroy   (gpointer                user_data);
+#endif
 static gboolean       thunar_application_show_dialogs           (gpointer                user_data);
 static void           thunar_application_show_dialogs_destroy   (gpointer                user_data);
 static void           thunar_application_process_files          (ThunarApplication      *application);
@@ -132,11 +133,13 @@ struct _ThunarApplication
 
   guint                  show_dialogs_timer_id;
 
-  GVolumeMonitor        *volume_monitor;
+#ifdef HAVE_GUDEV
+  GUdevClient           *udev_client;
 
   GSList                *volman_udis;
   guint                  volman_idle_id;
   guint                  volman_watch_id;
+#endif
 
   GList                 *files_to_launch;
 };
@@ -191,7 +194,10 @@ thunar_application_class_init (ThunarApplicationClass *klass)
 static void
 thunar_application_init (ThunarApplication *application)
 {
-  gchar *path;
+#ifdef HAVE_GUDEV
+  static const gchar *subsystems[] = { "block", "input", "usb", NULL };
+#endif
+  gchar              *path;
 
   /* initialize the application */
   application->preferences = thunar_preferences_get ();
@@ -208,13 +214,15 @@ thunar_application_init (ThunarApplication *application)
       g_free (path);
     }
 
-  /* connect to the volume manager */
-  application->volume_monitor = g_volume_monitor_get ();
+#ifdef HAVE_GUDEV
+  /* establish connection with udev */
+  application->udev_client = g_udev_client_new (subsystems);
 
-  /* connect the volume manager support callbacks (used to spawn thunar-volman appropriately) */
-  g_signal_connect (application->volume_monitor, "drive-connected", G_CALLBACK (thunar_application_drive_connected), application);
-  g_signal_connect (application->volume_monitor, "drive-disconnected", G_CALLBACK (thunar_application_drive_disconnected), application);
-  g_signal_connect (application->volume_monitor, "drive-eject-button", G_CALLBACK (thunar_application_drive_eject), application);
+  /* connect to the client in order to be notified when devices are plugged in
+   * or disconnected from the computer */
+  g_signal_connect (application->udev_client, "uevent", 
+                    G_CALLBACK (thunar_application_uevent), application);
+#endif
 }
 
 
@@ -238,6 +246,7 @@ thunar_application_finalize (GObject *object)
       g_free (path);
     }
 
+#ifdef HAVE_GUDEV
   /* cancel any pending volman watch source */
   if (G_UNLIKELY (application->volman_watch_id != 0))
     g_source_remove (application->volman_watch_id);
@@ -250,8 +259,9 @@ thunar_application_finalize (GObject *object)
   g_slist_foreach (application->volman_udis, (GFunc) g_free, NULL);
   g_slist_free (application->volman_udis);
 
-  /* disconnect from the volume monitor */
-  g_object_unref (application->volume_monitor);
+  /* disconnect from the udev client */
+  g_object_unref (application->udev_client);
+#endif
 
   /* drop any running "show dialogs" timer */
   if (G_UNLIKELY (application->show_dialogs_timer_id != 0))
@@ -464,123 +474,67 @@ thunar_application_window_destroyed (GtkWidget         *window,
 
 
 
+#ifdef HAVE_GUDEV
 static void
-thunar_application_drive_connected (GVolumeMonitor    *volume_monitor,
-                                    GDrive            *drive,
-                                    ThunarApplication *application)
+thunar_application_uevent (GUdevClient       *client,
+                           const gchar       *action,
+                           GUdevDevice       *device,
+                           ThunarApplication *application)
 {
-  gchar *udi = NULL;
+  const gchar *sysfs_path;
+  gboolean     is_cdrom = FALSE;
+  gboolean     has_media = FALSE;
+  GSList      *lp;
 
-  _thunar_return_if_fail (G_IS_VOLUME_MONITOR (volume_monitor));
-  _thunar_return_if_fail (application->volume_monitor == volume_monitor);
-  _thunar_return_if_fail (G_IS_DRIVE (drive));
+  _thunar_return_if_fail (G_UDEV_IS_CLIENT (client));
+  _thunar_return_if_fail (action != NULL && *action != '\0');
+  _thunar_return_if_fail (G_UDEV_IS_DEVICE (device));
   _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
+  _thunar_return_if_fail (client == application->udev_client);
 
-  /* determine the HAL UDI for this drive */
-  udi = g_drive_get_identifier (drive, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
+  /* determine the sysfs path of the device */
+  sysfs_path = g_udev_device_get_sysfs_path (device);
 
-  /* check if we have a UDI */
-  if (G_LIKELY (udi != NULL))
+  /* check if the device is a CD drive */
+  is_cdrom = g_udev_device_get_property_as_boolean (device, "ID_CDROM");
+  has_media = g_udev_device_get_property_as_boolean (device, "ID_CDROM_MEDIA");
+
+  /* distinguish between "add", "change" and "remove" actions, ignore "move" */
+  if (g_strcmp0 (action, "add") == 0  
+      || (is_cdrom && has_media && g_strcmp0 (action, "change") == 0))
     {
-      /* only insert the UDI if we don't have it already. free it otherwise */
-      if (g_slist_find_custom (application->volman_udis, udi, (GCompareFunc) g_utf8_collate) == NULL)
-        application->volman_udis = g_slist_prepend (application->volman_udis, udi);
-      else
-        g_free (udi);
+      /* only insert the path if we don't have it already */
+      if (g_slist_find_custom (application->volman_udis, sysfs_path, 
+                               (GCompareFunc) g_utf8_collate) == NULL)
+        {
+          application->volman_udis = g_slist_prepend (application->volman_udis, 
+                                                      g_strdup (sysfs_path));
+
+          /* check if there's currently no active or scheduled handler */
+          if (G_LIKELY (application->volman_idle_id == 0 
+                        && application->volman_watch_id == 0))
+            {
+              /* schedule a new handler using the idle source, which invokes the handler */
+              application->volman_idle_id = 
+                g_idle_add_full (G_PRIORITY_LOW, thunar_application_volman_idle, 
+                                 application, thunar_application_volman_idle_destroy);
+            }
+        }
     }
-
-  /* check if there's currently no active or scheduled handler */
-  if (G_LIKELY (application->volman_idle_id == 0 && application->volman_watch_id == 0))
+  else if (g_strcmp0 (action, "remove") == 0)
     {
-      /* schedule a new handler using the idle source, which invokes the handler */
-      application->volman_idle_id = g_idle_add_full (G_PRIORITY_LOW, thunar_application_volman_idle,
-                                                     application, thunar_application_volman_idle_destroy);
-    }
-}
-
-
-
-static void
-thunar_application_drive_disconnected (GVolumeMonitor    *volume_monitor,
-                                       GDrive            *drive,
-                                       ThunarApplication *application)
-{
-  GSList *lp;
-  gchar  *udi;
-
-  _thunar_return_if_fail (G_IS_VOLUME_MONITOR (volume_monitor));
-  _thunar_return_if_fail (application->volume_monitor == volume_monitor);
-  _thunar_return_if_fail (G_IS_DRIVE (drive));
-  _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
-
-  /* determine the HAL UDI for this drive */
-  udi = g_drive_get_identifier (drive, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
-
-  /* check if we have a UDI */
-  if (G_LIKELY (udi != NULL))
-    {
-      /* look for the UDI in the list of pending UDIs */
-      lp = g_slist_find_custom (application->volman_udis, udi, (GCompareFunc) g_utf8_collate);
+      /* look for the sysfs path in the list of pending paths */
+      lp = g_slist_find_custom (application->volman_udis, sysfs_path, 
+                                (GCompareFunc) g_utf8_collate);
 
       if (G_LIKELY (lp != NULL))
         {
-          /* free the UDI string */
+          /* free the sysfs path string */
           g_free (lp->data);
 
-          /* drop the UDI from the list of pending device UDIs */
+          /* drop the sysfs path from the list of pending device paths */
           application->volman_udis = g_slist_delete_link (application->volman_udis, lp);
         }
-
-      g_free (udi);
-    }
-}
-
-
-
-static void
-thunar_application_drive_eject (GVolumeMonitor    *volume_monitor,
-                                GDrive            *drive,
-                                ThunarApplication *application)
-{
-  GdkScreen *screen;
-  GError    *err = NULL;
-  gchar     *argv[4];
-  gchar     *udi;
-
-  _thunar_return_if_fail (G_IS_VOLUME_MONITOR (volume_monitor));
-  _thunar_return_if_fail (application->volume_monitor == volume_monitor);
-  _thunar_return_if_fail (G_IS_DRIVE (drive));
-  _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
-
-  /* determine the HAL UDI for this device */
-  udi = g_drive_get_identifier (drive, G_VOLUME_IDENTIFIER_KIND_HAL_UDI);
-
-  /* check if we have a UDI */
-  if (G_LIKELY (udi != NULL))
-    {
-      /* generate the argument list for exo-eject */
-      argv[0] = (gchar *) "exo-eject";
-      argv[1] = (gchar *) "-h";
-      argv[2] = (gchar *) udi;
-      argv[3] = NULL;
-
-      /* locate the currently active screen (the one with the pointer) */
-      screen = xfce_gdk_screen_get_active (NULL);
-
-      /* try to spawn the volume_monitor on the active screen */
-      if (!gdk_spawn_on_screen (screen, NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &err))
-        {
-          /* failed to launch exo-eject, inform the user about this */
-          thunar_dialogs_show_error (screen, err, _("Failed to execute \"%s\""), "exo-eject");
-          g_error_free (err);
-        }
-      else
-        {
-          /* we most probably removed the device */
-          thunar_application_drive_disconnected (volume_monitor, drive, application);
-        }
-
-      g_free (udi);
     }
 }
 
@@ -694,6 +648,7 @@ thunar_application_volman_watch_destroy (gpointer user_data)
 {
   THUNAR_APPLICATION (user_data)->volman_watch_id = 0;
 }
+#endif /* HAVE_GUDEV */
 
 
 
@@ -1369,7 +1324,7 @@ thunar_application_link_into (ThunarApplication *application,
   g_free (display_name);
 
   /* collect the target files and launch the job */
-  thunar_application_collect_and_launch (application, parent, "stock_link",
+  thunar_application_collect_and_launch (application, parent, "insert-link",
                                          title, thunar_io_jobs_link_files, 
                                          source_file_list, target_file, 
                                          new_files_closure);
@@ -1537,7 +1492,7 @@ thunar_application_unlink_files (ThunarApplication *application,
       if (G_LIKELY (response == GTK_RESPONSE_YES))
         {
           /* launch the "Delete" operation */
-          thunar_application_launch (application, parent, "stock_delete",
+          thunar_application_launch (application, parent, "edit-delete",
                                      _("Deleting files..."), unlink_stub,
                                      path_list, path_list, NULL);
         }
@@ -1572,7 +1527,7 @@ thunar_application_trash (ThunarApplication *application,
   _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
   _thunar_return_if_fail (file_list != NULL);
 
-  thunar_application_launch (application, parent, "gnome-fs-trash-full", 
+  thunar_application_launch (application, parent, "user-trash-full", 
                              _("Moving files into the trash..."), trash_stub,
                              file_list, NULL, NULL);
 }
@@ -1611,7 +1566,7 @@ thunar_application_creat (ThunarApplication *application,
   _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
   
   /* launch the operation */
-  thunar_application_launch (application, parent, "stock_new",
+  thunar_application_launch (application, parent, "document-new",
                              _("Creating files..."), creat_stub,
                              file_list, file_list, new_files_closure);
 }
@@ -1650,7 +1605,7 @@ thunar_application_mkdir (ThunarApplication *application,
   _thunar_return_if_fail (THUNAR_IS_APPLICATION (application));
 
   /* launch the operation */
-  thunar_application_launch (application, parent, "stock_folder",
+  thunar_application_launch (application, parent, "folder-new",
                              _("Creating directories..."), mkdir_stub,
                              file_list, file_list, new_files_closure);
 }
@@ -1714,7 +1669,7 @@ thunar_application_empty_trash (ThunarApplication *application,
       file_list.prev = NULL;
 
       /* launch the operation */
-      thunar_application_launch (application, parent, "gnome-fs-trash-empty",
+      thunar_application_launch (application, parent, "user-trash",
                                  _("Emptying the Trash..."),
                                  unlink_stub, &file_list, NULL, NULL);
 
